#!/usr/bin/python
# -*- coding: utf-8 -*-

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
# Author: Tomas "Tojaj" Mlcoch (xtojaj gmail dot com)
# Web: http://tojaj.com
#
# Copyright 2010 Tomas Mlcoch


import threading
import socket
import select
import platform
from impacket import ImpactDecoder, ImpactPacket

class ReceiverThread(threading.Thread):
    """Thread class for recieving icmp echo replies."""

    def __init__(self, model, lck):
        self.stop = False
        self.lock = lck
        self.model = model
        threading.Thread.__init__(self)

    def end(self):
        """Sends stop request."""
        self.stop = True

    def run(self):
        """Starts thread for recieve icmp echo replies."""
        s = socket.socket(socket.AF_INET, socket.SOCK_RAW, socket.IPPROTO_ICMP)
        if platform.system() == 'Windows':
            s.bind((socket.gethostbyname(socket.gethostname()), 0))
        s.setsockopt(socket.IPPROTO_IP, socket.IP_HDRINCL, 1)
        while 1:
            try:
                data = select.select([s], [], [], 1)[0]
            except select.error, err:
                if err[0] != 4: raise  # ignore Interrupted system call
                continue

            if s in data:
                echo = s.recvfrom(1024)
                pkt = ImpactDecoder.IPDecoder().decode(echo[0])
                pkt_icmp = pkt.child()

                if ImpactPacket.ICMP().ICMP_ECHOREPLY == pkt_icmp.get_icmp_type():
                    try:
                        self.lock.acquire()
                        self.model.refresh(pkt.get_ip_src(), pkt_icmp.get_icmp_id())
                    except (KeyError):
                        pass
                    finally:
                        self.lock.release()

            if self.stop:
                s.close()
                return
