#!/usr/bin/python
# -*- coding: utf-8 -*-

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
# Author: Tomas "Tojaj" Mlcoch (xtojaj gmail dot com)
# Web: http://tojaj.com
#
# Copyright 2010 Tomas Mlcoch


import threading
import socket
import time
import platform
from impacket import ImpactPacket

MIN_ROUND_TIME = 1.5  # sec
STOP_TIME = 0.4  # sec

class SenderThread(threading.Thread):
    """Thread class for sending icmp echo requests."""

    def __init__(self, model, lck):
        self.stop = False
        self.lock = lck
        self.model = model
        threading.Thread.__init__(self)

    def end(self):
        """Sends stop request."""
        self.stop = True

    def run(self):
        """Starts thread for send icmp echo requests."""
        icmp = ImpactPacket.ICMP()
        icmp.set_icmp_type(icmp.ICMP_ECHO)
        icmp.contains(ImpactPacket.Data("0"*64))

        s = socket.socket(socket.AF_INET, socket.SOCK_RAW, socket.IPPROTO_ICMP)
        s.setsockopt(socket.IPPROTO_IP, socket.IP_HDRINCL, 1)

        cycle_start = time.time()
        while 1:
            if self.stop:
                s.close()
                return

            ip = ImpactPacket.IP()
            if platform.system() == 'Windows':
                ip.set_ip_src(socket.gethostbyname(socket.gethostname()))

            self.lock.acquire()
            address, seq, new_cycle = self.model.next()
            self.lock.release()

            if not address:
                time.sleep(STOP_TIME)
                continue

            ip.set_ip_dst(address)
            ip.contains(icmp)

            icmp.set_icmp_id(seq)
            icmp.set_icmp_cksum(0)
            icmp.auto_checksum = 1

            s.sendto(ip.get_packet(), (address, 0))

            if new_cycle:
                rest = time.time() - cycle_start
                if (time.time() - cycle_start) < MIN_ROUND_TIME:
                    time.sleep(MIN_ROUND_TIME - rest)
                cycle_start = time.time()
            else:
                time.sleep(STOP_TIME)
