#!/usr/bin/python
# -*- coding: utf-8 -*-

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
# Author: Tomas "Tojaj" Mlcoch (xtojaj gmail dot com)
# Web: http://tojaj.com
#
# Copyright 2010 Tomas Mlcoch


import sys
import threading
import logging

import core.log
from core.settings import Settings
from core.datamodel import DataModel
from core.thread_receiver import ReceiverThread
from core.thread_sender import SenderThread
from core.gui_interface import GuiInterface

SUDO_NEED = """You do not have sufficient rights!
Run PyPinger as root (administrator).

Detail: You need permissions to create ICMP packet."""


def permission_test():
    """Test if program have sufficient rights for create ICMP packet."""
    import socket
    try:
        socket.socket(socket.AF_INET, socket.SOCK_RAW, socket.IPPROTO_ICMP)
    except socket.error:
        return False
    return True


if __name__ == "__main__":
    interface = GuiInterface()

    if not permission_test():
        interface.warning_msg(SUDO_NEED)
        sys.exit(1)  

    settings = Settings()
    settings.load()

    log = logging.getLogger("pypinger")
    if settings.logging:
        core.log.init()
    log.info("PyPinger starts with args: {0}".format(' '.join(sys.argv)))

    model = DataModel()
    lock = threading.Lock()

    settings.fill_model(model, lock)

    interface.set_model(model)
    interface.set_lock(lock)

    receiver = ReceiverThread(model, lock)
    receiver.start()
    sender = SenderThread(model, lock)
    sender.start()

    ret = interface.run()

    sender.end()
    sender.join()
    receiver.end()
    receiver.join()

    settings.save()

    log.info("PyPinger exits")
    
    sys.exit(ret)
